import java.awt.*;
import java.awt.geom.*;

/**
 * A router in a network.
 *
 * @author Eric Lemar
 */
public class RouterNode extends NetworkNode 
{
   protected Color color;
   /**The size of the graphic*/
   protected double size;
   public RouterNode()
   {
      color = Color.BLACK;
      size  = 20;
   }

  public void draw(Graphics2D g2) {
      //Borrowed from CircleNode
      Rectangle2D rect = new Rectangle2D.Double(
         x, y,size,size);
      Color oldColor = g2.getColor();
      g2.setColor(color);
      g2.fill(rect);
      g2.setColor(oldColor);
      g2.draw(rect);
  }

   public boolean contains(Point2D p)
   {
      Rectangle2D rect = new Rectangle2D.Double(
         x, y, size, size);
      return rect.contains(p);
   }


   public Rectangle2D getBounds()
   {
      return new Rectangle2D.Double(x, y, size, size);
   }

   public Point2D getConnectionPoint(Point2D other)
   {
      double centerX = x+size/2;
      double centerY = y+size/2;
      double dx = other.getX() - centerX;
      double dy = other.getY() - centerY;
      double distance = Math.sqrt(dx * dx + dy * dy);

      if (distance==0.0) {
         return other;
      }
  
      double angle;
      if (dy>0 ) {
        angle = Math.acos(dx/distance);
      } else {
        angle = 2*Math.PI-Math.acos(dx/distance);
      }

      if (angle<=(Math.PI/4) || angle>=(7*Math.PI/4)) {
        return new Point2D.Double(centerX+size/2,centerY);
      } else if (angle>=Math.PI/4 && angle<=3*Math.PI/4) {
        return new Point2D.Double(centerX,centerY+size/2);
      } else if (angle>=3*Math.PI/4 && angle<=5*Math.PI/4) {
        return new Point2D.Double(centerX-size/2,centerY);
      } else {
        return new Point2D.Double(centerX,centerY-size/2);
      }
   }


}
